--Character Randomizer (based on CR v3.1)
--Made by Nightwolf

local urmain = assert(_URANDOM,"URANDOM: variables.lua isn't loaded properly!")
local vars = urmain.var.charrandom

local skinlist_str=""
local skinlist_tab={}

//Netgame synchronization stuff
addHook("NetVars", function(net)
	skinlist_str = net(skinlist_str)
	skinlist_tab = net(skinlist_tab)
end)

//Returns the skinlist table index if it's in the skin table or nil if it's not found
local function skintabindex(skin)
	if #skinlist_tab == 0 then return nil end
	if not skin then return nil end
	for k,v in ipairs(skinlist_tab) do
		if v == skin then return k end
	end
	return nil
end

//Checks if the skin is valid
local function isskinvalid(skin)
	for sk in skins.iterate do
		if sk.name == skin then return true end
	end
	return false
end

//Make skin list string from the skinlist table
local function makeskinstr()
	skinlist_str = ""
	for k,v in ipairs(skinlist_tab) do
		if skinlist_str == "" then skinlist_str = v else skinlist_str = $1..","..v end
	end
end

//Get a list of skins from all available skins (called by 'cr_skin reload' and after loading the mod)
local function getskinlist(p)
	skinlist_tab = {}
	for sk in skins.iterate do
		if p then CONS_Printf(p,"CharRandom: Added skin "..sk.name) end
		table.insert(skinlist_tab,sk.name)
	end
	makeskinstr()
end

//Set up skin (used by givenewskin() function since v3, part of it before v3)
local function setupskin(p,skin)
	p.cr_skin = skin
	p.cr_time = vars.crtime * TICRATE
	if vars.timeoffset != 0 then 
		p.cr_offset = P_RandomRange(-vars.timeoffset*TICRATE,vars.timeoffset*TICRATE)
		if p.cr_time + p.cr_offset > 3600*TICRATE then p.cr_offset = 3600*TICRATE - p.cr_time
		elseif p.cr_time + p.cr_offset < TICRATE then p.cr_offset = TICRATE - p.cr_time end
		p.cr_time = $1 + p.cr_offset
	else
		p.cr_offset = 0
	end
	R_SetPlayerSkin(p,p.cr_skin)
	if p.mo.state >= S_PLAY_SUPER_TRANS1 and p.mo.state <= S_PLAY_SUPER_TRANS6 then p.mo.state = S_SPAWNSTATE end
end

//Give a new skin to a player
local function givenewskin(p)
	if #skinlist_tab < 2 then return end
	local isskinok=false
	local isfloat=false
	local rnum=0
	if p.secondjump == 1 and p.charability == CA_FLOAT then isfloat = true end
	//Get new current skin
	if not skintabindex(p.cr_next)
		while not isskinok do
			rnum=P_RandomRange(1,#skinlist_tab)
			if R_SkinUsable(p,skinlist_tab[rnum]) and skinlist_tab[rnum] != p.cr_skin then
				setupskin(p,skinlist_tab[rnum])
				isskinok=true
			end
		end
		if p.cr_next and vars.shownextskin == 1 then CONS_Printf(p,"Skin '"..p.cr_next.."' not found, selecting '"..p.cr_skin.."' instead") end
	else
		setupskin(p,p.cr_next)
	end
	//Get new next skin
	local isnextok=false
	while not isnextok do
		rnum=P_RandomRange(1,#skinlist_tab)
		if R_SkinUsable(p,skinlist_tab[rnum]) and skinlist_tab[rnum] != p.cr_skin then
			p.cr_next = skinlist_tab[rnum]
			isnextok=true
		end
	end
	if isfloat then 
		if p.charability != CA_FLOAT then p.secondjump = 0 end
	end
end

//Skin list management commands
COM_AddCommand("cr_skin", function(p,cmd,arg)
	if cmd == "list" then
		if skinlist_str == "" then CONS_Printf(p,"Skin list is empty!") else CONS_Printf(p,"Skin list: "..skinlist_str) end
	elseif cmd == "add" then
		if arg == nil then
			CONS_Printf(p,"\x82".."cr_skin add <skin name>\x80 - add a skin to the skin list")
		elseif isskinvalid(arg) then
			if skintabindex(arg) then
				CONS_Printf(p,"Skin '"..arg.."' is already on the skin list")
			else
				table.insert(skinlist_tab,arg)
				makeskinstr()
				CONS_Printf(p,"Skin '"..arg.."' added")
			end
		else
			CONS_Printf(p,"Skin '"..arg.."' is invalid")
		end
	elseif cmd == "remove" then
		if arg == nil then
			CONS_Printf(p,"\x82".."cr_skin remove <skin name>\x80 - remove a skin from the skin list")
		elseif isskinvalid(arg) then
			if skintabindex(arg) then
				table.remove(skinlist_tab,skintabindex(arg))
				makeskinstr()
				CONS_Printf(p,"Skin '"..arg.."' has been removed from the list")
			else
				CONS_Printf(p,"Skin '"..arg.."' is not on the skin list")
			end
		else
			CONS_Printf(p,"Skin '"..arg.."' is invalid")
		end
	elseif cmd == "reload" then
		getskinlist(p)
		CONS_Printf(p,"Skin list reloaded!")
	else
		CONS_Printf(p,"\x83".."cr_skin commands: ")
		CONS_Printf(p,"\x82".."cr_skin list\x80 - print the skin list")
		CONS_Printf(p,"\x82".."cr_skin add <skin name>\x80 - add a skin to the skin list")
		CONS_Printf(p,"\x82".."cr_skin remove <skin name>\x80 - remove a skin from the skin list")
		CONS_Printf(p,"\x82".."cr_skin reload\x80 - reload the skin list (add all available skins to the list)")
	end
end, 1)

//Checks if CR is enabled and if player has an object and counts down to 0
//If player changes their skin manually, it will be reverted to the randomly selected one
addHook("PlayerThink", function(p)
	if vars.isenabled == 0 or modeattacking then return end
	if #skinlist_tab < 2 then vars.isenabled = 0; print("Character randomizer has been disabled - not enough skins"); return end
	if not p.mo then return end
	if not P_IsValidSprite2(p.mo, p.mo.sprite2) then p.mo.sprite2 = SPR2_STND end
	if vars.sa2random == 1 then //If SA2 Rando mode is enabled
		if not p.cr_skin then givenewskin(p) end
		if p.mo.skin != p.cr_skin then R_SetPlayerSkin(p,p.cr_skin) end
		return 
	end
	if not p.cr_offset then p.cr_offset = 0 end
	if p.cr_gamemap != gamemap then p.cr_gamemap = gamemap end
	if p.exiting > 0 then p.cr_time = 0; return end
	if p.mo.health < 1 then p.cr_time = 0; return end
	if mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS then return end
	if not p.cr_skin or not p.cr_time then givenewskin(p) end
	p.cr_time = (p.cr_time or vars.crtime * TICRATE) - 1
	if p.cr_offset > 0 and vars.timeoffset > 0 then
		if p.cr_offset > vars.timeoffset then p.cr_offset = vars.timeoffset*TICRATE end
		if p.cr_time > (vars.crtime * TICRATE) + p.cr_offset then p.cr_time = vars.crtime * TICRATE + p.cr_offset end
	else
		if p.cr_time > vars.crtime * TICRATE then p.cr_time = vars.crtime * TICRATE end
	end
	if p.cr_time < 1 or vars.ultimate == 1 then givenewskin(p) end
	if p.mo.skin != p.cr_skin then R_SetPlayerSkin(p,p.cr_skin) end
end)

//If SA2 Rando mode (only change character on level change) is enabled and map changes, then change skin
addHook("MapLoad", function(mapnum)
	if modeattacking then return end
	if vars.isenabled == 0 or vars.sa2random == 0 then return end
	if mapheaderinfo[mapnum].typeoflevel & TOL_NIGHTS then return end
	for p in players.iterate do
		if not p.cr_gamemap then p.cr_gamemap = -1 end
		if p.cr_gamemap != mapnum then //Prevent SP deaths from changing skin
			givenewskin(p)
			p.cr_gamemap = mapnum
		end
	end
end)

//Draws the countdown clock, but checks the same stuff as "PlayerThink" hook before drawing it
hud.add(function(v,p,cam)
	if vars.sa2random == 1 or vars.showtimer == 0 then return end
	if vars.isenabled == 0 or modeattacking then return end
	if not p.mo then return end
	if p.exiting > 0 return end
	if p.mo.health < 1 then return end
	if mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS then return end
	if not p.cr_skin or not p.cr_time then return end
	if vars.ultimate == 0 then
		v.timemin = G_TicsToMinutes(p.cr_time,true)
		v.timesec = G_TicsToSeconds(p.cr_time)
		v.timecsec = G_TicsToCentiseconds(p.cr_time)
		if p.cr_time < TICRATE*vars.crtime/10 then
			v.timestr = string.format("Skin time left:\x85 %d:%02d.%02d",v.timemin,v.timesec,v.timecsec)
		elseif p.cr_time < TICRATE*vars.crtime/4 then 
			v.timestr = string.format("Skin time left:\x82 %d:%02d.%02d",v.timemin,v.timesec,v.timecsec)
		else
			v.timestr = string.format("Skin time left:\x83 %d:%02d.%02d",v.timemin,v.timesec,v.timecsec)
		end
		if p.cr_time < 1 then v.timestr = "Skin time left: \x85".."0:00.00" end
		v.drawString(300,10,v.timestr,V_SNAPTORIGHT|V_PERPLAYER,"right")
		if vars.shownextskin == 1 and p.cr_next then v.drawString(300,20,"Next skin: \x86"..p.cr_next,V_SNAPTORIGHT|V_PERPLAYER,"right") end
	else
		v.drawString(300,10,"\x85".."IDENTITY CRISIS MODE",V_SNAPTORIGHT|V_PERPLAYER,"right")
	end
end, "game")

urmain.cr_putskinsifnone = function(p)
	if #skinlist_tab == 0 then getskinlist(p) end
end
