--Misc. Randomizer (options that don't belong to other categories)
--Made by Nightwolf

local urmain = assert(_URANDOM,"URANDOM: variables.lua isn't loaded properly!")
local vars = urmain.var.miscrandom

local hightexturelist = {}
local lowtexturelist = {}
local flatflist = {}
local flatclist = {}

local shufflearray = urmain.shufflearray

//Banned linedef specials from texture randomization
local bannedlinespec = {
	[4] = true,
	[9] = true,
	[14] = true,
	[15] = true,
	[334] = true,
	[335] = true,
	[336] = true,
	[405] = true,
	[413] = true,
	[414] = true,
	[425] = true,
	[434] = true,
	[442] = true,
	[443] = true,
	[447] = true,
	[455] = true,
	[461] = true,
	[463] = true,
	[606] = true
}

//Vanilla sky numbers (excludes skies with numbers > 1000)
local skynums = {
	1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,
	21,22,29,30,31,33,34,35,40,50,51,52,53,54,55,56,57,
	58,59,64,66,68,98,99,100,103,107,117,127,128,129,
	132,159,172,300,301,302
}

addHook("NetVars",function(net)
	hightexturelist = net($)
	lowtexturelist = net($)
	flatflist = net($)
	flatclist = net($)
end)

local function isvarintab(tab,var)
	for k,v in pairs(tab) do
		if v == var then return true end
	end
	return false
end

local function insertTexture(tab,texture)
	if texture > 0 then
		table.insert(tab,texture)
	end
end

local function insertFlat(tab,flat)
	if flat ~= "F_SKY1" then
		table.insert(tab,flat)
	end
end

addHook("MapLoad",function(mapnum)
	flatflist = {}
	flatclist = {}
	hightexturelist = {}
	lowtexturelist = {}
	if vars.colorrand == 1 then
		for p in players.iterate do
			p.colrandcolor = P_RandomRange(1,#skincolors-1)
		end
	end
	if vars.abilityrand == 1 then
		for p in players.iterate do
			p.abrandability = P_RandomRange(CA_NONE,CA_TWINSPIN)
		end
	end
	if vars.texturerand > 0 then
		for side in sides.iterate do
			if bannedlinespec[side.special] then continue end
			if vars.texturerand == 1 then
				if side.toptexture > 0 then insertTexture(hightexturelist,side.toptexture) end
				if side.bottomtexture > 0 then insertTexture(lowtexturelist,side.bottomtexture) end
			else
				if side.toptexture > 0 and not isvarintab(hightexturelist,side.toptexture) then insertTexture(hightexturelist,side.toptexture) end
				if side.bottomtexture > 0 and not isvarintab(lowtexturelist,side.bottomtexture) then insertTexture(lowtexturelist,side.bottomtexture) end
			end
		end
		local htabindex = 1
		local ltabindex = 1
		if vars.texturerand == 1 then
			shufflearray(hightexturelist,#hightexturelist)
			shufflearray(lowtexturelist,#lowtexturelist)
		end
		for side in sides.iterate do
			if bannedlinespec[side.special] then continue end
			local htex = side.toptexture
			local ltex = side.bottomtexture
			if vars.texturerand == 1 then
				if htex > 0 then
					side.toptexture = hightexturelist[htabindex]
					htabindex = $ + 1
				end
				if ltex > 0 then
					side.bottomtexture = lowtexturelist[ltabindex]
					ltabindex = $ + 1
				end
			else
				local rnum
				if htex > 0 then
					rnum = urmain.randomRange(1,#hightexturelist)
					side.toptexture = hightexturelist[rnum]
				end
				if ltex > 0 then
					rnum = urmain.randomRange(1,#lowtexturelist)
					side.bottomtexture = lowtexturelist[rnum]
				end
			end
		end
	end
	if vars.flatrand > 0 then
		for sec in sectors.iterate do
			if vars.flatrand == 1 then
				insertFlat(flatflist,sec.floorpic)
				insertFlat(flatclist,sec.ceilingpic)
			else
				if not isvarintab(flatflist,sec.floorpic) then insertFlat(flatflist,sec.floorpic) end
				if not isvarintab(flatclist,sec.ceilingpic) then insertFlat(flatclist,sec.ceilingpic) end
			end
		end
		local ftabindex = 1
		local ctabindex = 1
		if vars.flatrand == 1 then
			shufflearray(flatflist,#flatflist)
			shufflearray(flatclist,#flatclist)
		end
		for sec in sectors.iterate do
			local sfpic = sec.floorpic
			local scpic = sec.ceilingpic
			if vars.flatrand == 1 then
				if sfpic ~= "F_SKY1" and sfpic ~= "-" then
					sec.floorpic = flatflist[ftabindex]
					ftabindex = $1 + 1
				end
				if scpic ~= "F_SKY1" and scpic ~= "-" then
					sec.ceilingpic = flatclist[ctabindex]
					ctabindex = $1 + 1
				end
			else
				local rnum
				if sfpic ~= "F_SKY1" and sfpic ~= "-" then
					rnum = urmain.randomRange(1,#flatflist)
					sec.floorpic = flatflist[rnum]
				end
				if scpic ~= "F_SKY1" and scpic ~= "-" then
					rnum = urmain.randomRange(1,#flatclist)
					sec.ceilingpic = flatclist[rnum]
				end
			end
		end
	end
	if vars.brightrand > 0 then
		for sec in sectors.iterate do
			if vars.brightrand == 1 then
				sec.lightlevel = $1 + urmain.randomRange(-64,64)
			else
				sec.lightlevel = urmain.randomRange(0,255)
			end
		end
	end
	if vars.skyrand == 1 then
		P_SetupLevelSky(skynums[urmain.randomRange(1,#skynums)])
	end
end)

addHook("PlayerThink",function(p)
	if vars.colorrand == 1 and p.colrandcolor then
		p.skincolor = p.colrandcolor
		if p.mo and not (p.powers[pw_super] or p.powers[pw_shield] & SH_FIREFLOWER) then p.mo.color = p.colrandcolor end
	end
	if vars.abilityrand == 1 and p.abrandability then
		p.charability = p.abrandability
	end
end)
