--Object Randomizer Remake (v2)
--Made by Nightwolf

local urmain = assert(_URANDOM,"URANDOM: variables.lua isn't loaded properly!")
local vars = urmain.var.objrandom
local objdefs = assert(_urandomobjdefs,"URANDOM: Object definitions are not present!")

local OR_InstantMove
--Compatibility with both 2.2.10 and 2.2.11
if VERSION == 202 and SUBVERSION <= 10 then
	OR_InstantMove = P_TeleportMove --Version 2.2.10 and older
else
	OR_InstantMove = P_SetOrigin --Version 2.2.11 and newer
end

--A table shuffling algorithm
local shufflearray = urmain.shufflearray

local objLoading = false

freeslot("MT_ULTRAND_OBJRAND_CHECKER")
mobjinfo[MT_ULTRAND_OBJRAND_CHECKER] = {
        doomednum = -1,
        spawnstate = S_INVISIBLE,
        spawnhealth = 999,
        seestate = S_NULL,
        seesound = sfx_None,
        reactiontime = 0,
        attacksound = sfx_None,
        painstate = S_NULL,
        painchance = 0,
        painsound = sfx_None,
        meleestate = S_NULL,
        missilestate = S_NULL,
        deathstate = S_NULL,
        xdeathstate = S_NULL,
        deathsound = sfx_None,
        speed = 1,
        radius = FRACUNIT,
        height = FRACUNIT,
        dispoffset = 0,
        mass = 1000,
        damage = 0,
        activesound = sfx_None,
        flags = 0,
        raisestate = 0
}
local orchecker = nil //MT_ULTRAND_OBJRAND_CHECKER object on NoReload maps

addHook("NetVars",function(net)
	objLoading = net($)
	orchecker = net($)
end)

--Table with Object Type randomization list addition functions
local addObject = {
[0] = nil,
[1] = function(mo,ttab)
	if objdefs.limited[mo.type] then
		table.insert(ttab,{mo,objdefs.limitedlist[urmain.randomRange(1,#objdefs.limitedlist)]})
	end
end,
[2] = function(mo,ttab,otpairs)
	if objdefs.full[mo.type] then
		table.insert(ttab,{mo,objdefs.fulllist[otpairs[mo.type]]})
	end
end,
[3] = function(mo,ttab)
	if objdefs.smart.enemies[mo.type] then
		table.insert(ttab,{mo,objdefs.smartlist.enemies[urmain.randomRange(1,#objdefs.smartlist.enemies)]})
	elseif objdefs.smart.pickups[mo.type] then
		table.insert(ttab,{mo,objdefs.smartlist.pickups[urmain.randomRange(1,#objdefs.smartlist.pickups)]})
	elseif objdefs.smart.scenery[mo.type] then
		table.insert(ttab,{mo,objdefs.smartlist.scenery[urmain.randomRange(1,#objdefs.smartlist.scenery)]})
	elseif objdefs.smart.rest[mo.type] then
		table.insert(ttab,{mo,objdefs.smartlist.rest[urmain.randomRange(1,#objdefs.smartlist.rest)]})
	end
end,
[4] = function(mo,ttab)
	if objdefs.full[mo.type] then
		table.insert(ttab,{mo,objdefs.fulllist[urmain.randomRange(1,#objdefs.fulllist)]})
	end
end,
}

local function randomizeObjects()
	objLoading = true
	
	objdefs.smart.pickups[MT_RING] = vars.randomrings
	objdefs.full[MT_RING] = vars.randomrings
	objdefs.smart.pickups[MT_BLUESPHERE] = vars.randomrings
	objdefs.full[MT_BLUESPHERE] = vars.randomrings
	
	local objpostab = {}
	local objpsmotab = {}
	local objtypetab = {}
	local objtypepairs = {} //Used for object type: uniform
	if vars.randomtype == 2 then
		local temppairs = {}
		for k,v in ipairs(objdefs.fulllist) do
			table.insert(temppairs,v)
		end
		shufflearray(temppairs,#temppairs)
		for k,v in ipairs(temppairs)
			objtypepairs[v] = k
		end
	end
	for mo in mobjs.iterate() do
		if objdefs.full[mo.type] and mo.type ~= MT_SPIKE then
			table.insert(objpsmotab,mo)
			table.insert(objpostab,{mo.x,mo.y,mo.z})
		end
		if vars.randomtype > 0 then addObject[vars.randomtype](mo,objtypetab,objtypepairs) end
	end
	if vars.setposition == 1 then
		shufflearray(objpostab,#objpostab)
		for k,v in ipairs(objpostab) do
			OR_InstantMove(objpsmotab[k],v[1],v[2],v[3])
		end
	elseif vars.setposition == 2 then
		shufflearray(objpostab,#objpostab)
		for k,v in ipairs(objpsmotab) do
			local isdone = false
			local nx,ny
			while not isdone do
				nx = urmain.randomRange(-32768,32767)*FRACUNIT
				ny = urmain.randomRange(-32768,32767)*FRACUNIT
				if R_PointInSubsectorOrNil(nx,ny) and P_CheckPosition(v,nx,ny) then isdone = true end
			end
			OR_InstantMove(v,nx,ny,v.z)
			v.z = v.floorz
			if v.type == MT_RING and v.flags2 & MF2_AMBUSH then v.z = $1 + 24*FRACUNIT end
			if v.spawnpoint then 
				v.spawnpoint.x = FixedInt(nx)
				v.spawnpoint.y = FixedInt(ny)
				v.spawnpoint.z = 0
			end
		end
	end
	if vars.randomsize > 0 then
		local rmin = FRACUNIT
		local rmax = FRACUNIT
		for f = 1,#objpsmotab do
			if vars.randomsize == 1 then 
				rmin = FRACUNIT/2
				rmax = 2*FRACUNIT
			else
				rmin = FRACUNIT/8
				rmax = 5*FRACUNIT
			end
			if urmain.randomChance(FRACUNIT/2) then
				objpsmotab[f].scale = urmain.randomRange(rmin>>4,FRACUNIT>>4)<<4 //P_RandomRange has range limit of 65536
			else
				objpsmotab[f].scale = urmain.randomRange(FRACUNIT>>4,rmax>>4)<<4
			end
		end
	end
	if vars.randomtype > 0 then
		for k,v in ipairs(objtypetab) do
			if v[1].valid then
				local nmobj = P_SpawnMobj(v[1].x,v[1].y,v[1].z,v[2])
				nmobj.scale = v[1].scale
				nmobj.angle = v[1].angle
				nmobj.state = S_SPAWNSTATE
				v[1].flags2 = $ | MF2_DONTRESPAWN
				P_KillMobj(v[1])
			end
		end
	end
	objLoading = false
end

addHook("MapChange",function(mapnum)
	orchecker = nil
end)

addHook("MapLoad",function(mapnum)
	if not vars.isenabled then return end
	if not multiplayer and mapheaderinfo[mapnum].levelflags & LF_NORELOAD then
		orchecker = P_SpawnMobj(0,0,0,MT_ULTRAND_OBJRAND_CHECKER)
	else
		orchecker = nil
	end
	if modeattacking or maptol & TOL_NIGHTS or maptol & TOL_XMAS then return end
	randomizeObjects()
end)

--If object type randomizing is enabled, give players 2 seconds of invincibility after respawn
addHook("PlayerSpawn", function(p)
	if not netgame and vars.isenabled then
		if modeattacking or maptol & TOL_NIGHTS or maptol & TOL_XMAS then return end --disable on NIGHTS levels
		if (not multiplayer and mapheaderinfo[gamemap].levelflags & LF_NORELOAD) and orchecker and not orchecker.valid then
			urmain.resetRNG() --To make sure objects get randomized according to the random seed
			randomizeObjects() 
		end
		p.powers[pw_flashing] = 2*TICRATE
	end
end)
