--Ultimate Randomizer Variables
--Made by Nightwolf

//Seeded RNG (used for object randomizer, texture/flat randomization and brightness randomization)
local urrng = newXRNG()

//Map Randomizer's seeded RNG, independent from the main one
local mrrng = newXRNG()

//Map randomizer seed offset (0 if none)
local mroffset = 0

local urandom = {}
urandom.lastCfg = "default" //Last config file name without extension
urandom.var = {} //Variables
urandom.cmds = {} //Commands
urandom.randomRange = function(a,b)
	if not urandom.var.main.seed then
		return P_RandomRange(a,b)
	else
		return urrng:randomRange(a,b)
	end
end
urandom.randomChance = function(p)
	if not urandom.var.main.seed then
		return P_RandomChance(p)
	else
		return urrng:randomChance(p)
	end
end
urandom.randomSeed = function(seed)
	urrng:setSeed(seed)
	urandom.var.main.seed = seed
	mroffset = 0
end
urandom.resetRNG = function()
	urrng:setSeed(urandom.var.main.seed)
end
urandom.shufflearray = function(tab,size) 
	if not size then size = #tab end
	for i = size,1,-1 do
		local index = urandom.randomRange(1,i)
		tab[i],tab[index] = $2,$1
	end
	return tab
end

//Map Randomizer RNG functions
urandom.resetmrrng = function()
	if mroffset == 0 then
		mrrng:setSeed(urandom.var.main.seed)
	else
		mrrng:setSeed(urandom.var.main.seed..mroffset)
	end
	mroffset = $ + 1
end
urandom.randomRangeMapRand = function(a,b)
	if not urandom.var.main.seed then
		return P_RandomRange(a,b)
	else
		return mrrng:randomRange(a,b)
	end
end
urandom.maprandshuffle = function(tab,size)
	if not size then size = #tab end
	for i = size,1,-1 do
		local index = urandom.randomRangeMapRand(1,i)
		tab[i],tab[index] = $2,$1
	end
	return tab
end

urandom.var.main = {}
urandom.var.charrandom = {}
urandom.var.objrandom = {}
urandom.var.maprandom = {}
urandom.var.physrandom = {}
urandom.var.miscrandom = {}

urandom.setupVariables = function()
	//Main Variables
	urandom.var.main.seed = 0 //Current seed (0 is fully random, SRB2 RNG is always used in this case)

	//Character Randomizer Variables
	urandom.var.charrandom.isenabled = 0
	urandom.var.charrandom.crtime = 30 //Time between skin changes
	urandom.var.charrandom.ultimate = 0 //Is ultimate on or off
	urandom.var.charrandom.sa2random = 0 //Is SA2 Rando-like mode on or off
	urandom.var.charrandom.timeoffset = 0 //Maximum crtime offset
	urandom.var.charrandom.shownextskin = 0 //Show the next skin?
	urandom.var.charrandom.showtimer = 1 //Show the timer?

	//Object Randomizer Variables
	urandom.var.objrandom.isenabled = 0
	urandom.var.objrandom.setposition = 0 //0-none, 1-swap, 2-randomize
	urandom.var.objrandom.randomtype = 0 //0-disable, 1-limited, 2-uniform, 3-smart, 4-full
	urandom.var.objrandom.randomsize = 0 //0-disable, 1-50 to 200%, 2-12.5 to 500%
	urandom.var.objrandom.randomrings = 0 //Include rings and blue spheres in object randomization

	//Map Randomizer Variables
	urandom.var.maprandom.isenabled = 0
	urandom.var.maprandom.maps = {} //Map list (global)
	urandom.var.maprandom.bossmaps = 1 //Allow boss maps
	urandom.var.maprandom.specialstages = 0 //Allow special stages
	urandom.var.maprandom.autorandom = 1 //Automatic randomization
	urandom.var.maprandom.mapnum = 1 //Current map number (in the map list, not SRB2 map number)

	//Physics Randomizer Variables
	urandom.var.physrandom.isenabled = 0
	urandom.var.physrandom.phtime = 30 //Time between physics changes
	urandom.var.physrandom.speed = 1 //Speed randomization (0-off,1-limited,2-full)
	urandom.var.physrandom.spindash = 0 //Spindash speed randomization (0-off,1-limited,2-full)
	urandom.var.physrandom.friction = 0 //Friction randomization (0-off,1-limited,2-full)
	urandom.var.physrandom.jumping = 0 //Jump height randomization (0-off,1-limited,2-full)
	urandom.var.physrandom.actionspd = 0 //Action Speed randomization (0-off,1-limited,2-full)
	urandom.var.physrandom.showtimer = 1 //Show the timer?

	//Misc. Randomizer Variables
	urandom.var.miscrandom.texturerand = 0 //Wall Texture randomization (0-off,1-swap,2-randomize)
	urandom.var.miscrandom.flatrand = 0 //Flat randomization (0-off,1-swap,2-randomize)
	urandom.var.miscrandom.brightrand = 0 //Brightness randomization (0-off,1-relative,2-absolute)
	urandom.var.miscrandom.colorrand = 0 //Color randomization per map (0-off,1-on)
	urandom.var.miscrandom.abilityrand = 0 //Ability randomization per map (0-off,1-on)
	urandom.var.miscrandom.skyrand = 0 //Sky texture randomization per map (0-off,1-on)
end

urandom.setupVariables()

rawset(_G,"_URANDOM",urandom)

addHook("MapChange",function(mapnum)
	urandom.resetRNG()
end)

addHook("NetVars",function(net)
	urrng.s = net(urrng.s)
	mrrng.s = net(mrrng.s)
	mroffset = net(mroffset)
end)
